/*---------------------------------------------------------------------------*\
 ##   ####  ######     |
 ##  ##     ##         | Copyright: ICE Stroemungsfoschungs GmbH
 ##  ##     ####       |
 ##  ##     ##         | http://www.ice-sf.at
 ##   ####  ######     |
-------------------------------------------------------------------------------
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2008 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is based on OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    Foam::gameOfLifeFunction

Description

SourceFiles
    gameOfLifeFunction.C

Contributors/Copyright:
    2014 Bernhard F.W. Gschaider <bgschaid@ice-sf.at>

 SWAK Revision: $Id$
\*---------------------------------------------------------------------------*/

#ifndef gameOfLifeFunction_H
#define gameOfLifeFunction_H

#include "FieldValuePluginFunction.H"

#include "HashSet.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class gameOfLifeFunction Declaration
\*---------------------------------------------------------------------------*/
class gameOfLifeFunction
:
    public FieldValuePluginFunction
{
    //- Disallow default bitwise assignment
    void operator=(const gameOfLifeFunction &);

    gameOfLifeFunction(const gameOfLifeFunction &);

    const scalar threshold=0.5;

    autoPtr<volScalarField> oldState_;

    void setConwayNumbers();

protected:

    bool isAlive(double val) { return val>threshold; }

    const volScalarField &oldState() { return oldState_(); }

    HashSet<label> survivingNumbers_;
    HashSet<label> birthNumbers_;

    autoPtr<volScalarField> doStep(const volScalarField &old);

    virtual void setArgument(
        label index,
        const string &content,
        const CommonValueExpressionDriver &driver
    );

    void doEvaluation();

public:

    // for sub-classes
    gameOfLifeFunction(
        const FieldValueExpressionDriver &parentDriver,
        const word &name,
        const string &arguments
    );

    // the actual function
    gameOfLifeFunction(
        const FieldValueExpressionDriver &parentDriver,
        const word &name
    );

    virtual ~gameOfLifeFunction() {}

    TypeName("gameOfLifeFunction");

};



// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
